/*******************************************************************************
 * Copyright (C) 2020 Team Loadcoder
 * 
 * This file is part of Loadcoder.
 * 
 * Loadcoder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Loadcoder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
package loadtests.cluster;

import com.loadcoder.cluster.clients.docker.LoadcoderCluster;
import com.loadcoder.cluster.clients.influxdb.InfluxDBClient;
import com.loadcoder.load.LoadUtility;
import com.loadcoder.load.scenario.ExecutionBuilder;
import com.loadcoder.load.scenario.Load;
import com.loadcoder.load.scenario.LoadBuilder;
import com.loadcoder.load.scenario.LoadScenario;
import static com.loadcoder.statics.Statics.*;
import static org.junit.Assert.assertTrue;

import org.junit.Test;
public class SimpleLoad {

	
	@Test
	public void main() {
		LoadScenario ls = new LoadScenario() {

			@Override
			public void loadScenario() {
				load("sleep1", () -> { LoadUtility.sleep(LoadUtility.random(40, 70));
				}).perform();
				load("sleep2", () -> { LoadUtility.sleep(LoadUtility.random(80, 90));
				}).perform();
				
				load("sometimesFails", () -> { 
					if(LoadUtility.random(0, 5) ==0) {
						throw new RuntimeException("wrong number");
					}
					LoadUtility.sleep(LoadUtility.random(20, 110));
				}).perform();
				
			}
		};

		Load l = new LoadBuilder(ls)
				.amountOfThreads(5)
				.rampup(20 * SECOND)
				.stopDecision(duration(600 * SECOND)).throttle(2, PER_SECOND, PER_THREAD).build();
		new ExecutionBuilder(l).storeAndConsumeResultRuntime(InfluxDBClient
				.setupInfluxDataConsumer(new LoadcoderCluster(), "SimpleTests", "SimpleLoadTest"))
				.build().execute().andWait();
	}
}
