package loadtests.server;

import static com.loadcoder.statics.Statics.*;
import static loadtests.tools.ClientUtils.createURLWithPort;

import org.springframework.http.HttpStatus;
import org.springframework.web.client.RestTemplate;

import com.loadcoder.load.chart.logic.ResultChart;
import com.loadcoder.load.chart.logic.RuntimeChart;
import com.loadcoder.load.scenario.ExecutionBuilder;
import com.loadcoder.load.scenario.Load;
import com.loadcoder.load.scenario.LoadBuilder;
import com.loadcoder.load.scenario.LoadScenarioTyped;
import com.loadcoder.result.Result;

public class StandAloneServerLoadTest {

	public static void main(String[] args) {
		
		LoadScenarioTyped<RestTemplate> loadScenario = new LoadScenarioTyped<RestTemplate>() {

			@Override
			public void loadScenario(RestTemplate t) {

				load("greeting", () -> {
					return t.getForEntity(createURLWithPort("/api/greeting?name=Loadcoder", 7060), String.class);
				}).handleResult(resultHandler -> {
					if (!resultHandler.getResponse().getStatusCode().equals(HttpStatus.OK)) {
						resultHandler.setStatus(false);
					}
				}).perform();
			}

			@Override
			public RestTemplate createInstance() {
				return new RestTemplate();
			}
		};
		
		Load load = new LoadBuilder(loadScenario).amountOfThreads(5).rampup(10 * SECOND)
				.stopDecision(duration(20 * SECOND)).build();

		RuntimeChart runtimeChart = new RuntimeChart();
		Result result = new ExecutionBuilder(load)
				.storeAndConsumeResultRuntime(runtimeChart)
				.build().execute().andWait()
				.getResultFromMemory();

		result.summaryStandard().build().prettyPrint();
		
		new ResultChart(result).waitUntilClosed();
	}
	
}
