package loadtests.examples.loadcoder;

import static com.loadcoder.statics.LogbackLogging.getLatestResultFile;
import static com.loadcoder.statics.LogbackLogging.getNewLogDir;
import static com.loadcoder.statics.LogbackLogging.setResultDestination;
import static com.loadcoder.statics.Statics.PER_MINUTE;
import static com.loadcoder.statics.Statics.SECOND;
import static com.loadcoder.statics.Statics.SHARED;
import static com.loadcoder.statics.Statics.duration;
import static com.loadcoder.statics.Statics.iterations;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.loadcoder.load.LoadUtility;
import com.loadcoder.load.chart.logic.ResultChart;
import com.loadcoder.load.scenario.ExecutionBuilder;
import com.loadcoder.load.scenario.Load;
import com.loadcoder.load.scenario.LoadBuilder;
import com.loadcoder.load.scenario.LoadScenario;
import com.loadcoder.result.Result;

/**
 * Examples of how to build and configure the Load part of a Loadcoder test.
 */
public class LoadExample {
	
	Logger log = LoggerFactory.getLogger(this.getClass()); 
	public static void main(String[] args) {
		new LoadExample().simpleLoadTest();
	}
	
	LoadScenario loadScenario = new LoadScenario() {

		@Override
		public void loadScenario() {

			load("sleep", () -> {
				LoadUtility.sleep(20);
			}).perform();
		}

	};
	
	public LoadScenario getScenario() {
		return loadScenario;
	}
	
	public void simpleLoadTest() {

		// Giving the test a unique directory for the result logs
		setResultDestination(getNewLogDir("target", "myLoadTest"));

		log.info("info meddelande");
		log.error("error meddelande");
		
		Load oneIterationOneThread = new LoadBuilder(getScenario()).build();
				
		
		Load durationBased = new LoadBuilder(getScenario())
				.stopDecision(duration(10 * SECOND))
				.build();
		
		Load iterationBased = new LoadBuilder(loadScenario)
				.stopDecision(iterations(20))
				.build();
		
		Load throttleTransactions = new LoadBuilder(loadScenario)
				.amountOfThreads(10)
				.rampup(10 * SECOND)
				.stopDecision(iterations(20))
				.throttle(180, PER_MINUTE, SHARED)
				.build();
		
		Load throttleIteration = new LoadBuilder(loadScenario)
				.amountOfThreads(10)
				.rampup(5 * SECOND)
				.stopDecision(iterations(10))
				.throttleIterations(120, PER_MINUTE, SHARED)
				.build();
		
		Result result = new ExecutionBuilder(throttleIteration).build().execute()
				.andWait().getReportedResultFromResultFile();

		result.summaryStandard().build().prettyPrint();
	}
	
	public void openResultFromLatestLogDir() {
		

		Result result = new Result(getLatestResultFile("target", "myLoadTest"));
		result.summaryStandard().build().prettyPrint();
		
		ResultChart chart = new ResultChart(result);
		chart.waitUntilClosed();
	}
}