package loadtests.examples.loadcoder;

import static com.loadcoder.statics.Statics.MINUTE;
import static com.loadcoder.statics.Statics.PER_SECOND;
import static com.loadcoder.statics.Statics.SECOND;
import static com.loadcoder.statics.Statics.SHARED;
import static com.loadcoder.statics.Statics.duration;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.loadcoder.load.LoadUtility;
import com.loadcoder.load.chart.logic.ResultChart;
import com.loadcoder.load.chart.logic.RuntimeChart;
import com.loadcoder.load.scenario.ExecutionBuilder;
import com.loadcoder.load.scenario.Load;
import com.loadcoder.load.scenario.LoadBuilder;
import com.loadcoder.load.scenario.LoadScenario;
import com.loadcoder.load.scenario.ResultModel;
import com.loadcoder.result.Result;

/**
 * Examples for how to contruct the transactions the load method inside the LoadScenario body
 */
public class TransactionExamples {

	Logger log = LoggerFactory.getLogger(this.getClass());
	
	public static void main(String[] args) {
		new TransactionExamples().simpleLoadTest();
	}
	
	LoadScenario loadScenario = new LoadScenario() {

		@Override
		public void loadScenario() {

			load("nothingToReturn", () -> {
			}).perform();
			
			String securityToken = load("getSecurityToken", () -> {
				return getSecurityToken();
			}).perform();
			
			load("authenticate", () -> {
				authenticate(securityToken);
			}).perform();
			
			
			load("handleResult", () -> {
				return new SomeKindOfObject();
			})
			.handleResult(resultHandler->{
				SomeKindOfObject returnedObject = resultHandler.getResponse();
				if(returnedObject == null) {
					resultHandler.setStatus(false);
				}
				
				if(resultHandler.getResponseTime() > 500) {
					resultHandler.setStatus(false);
					log.error(resultHandler.getTransactionName() + " very slow");
				}
				resultHandler.setMessage("message that will be included in the result log");
			})
			.perform();
			
			
			load("throwException", () -> {
				throw new IOException("Transaction threw exception");
			}).handleResult(resultHandler->{
				
				Exception theThrownException = resultHandler.getException();
				if(theThrownException instanceof RuntimeException) {
					resultHandler.setStatus(true);
				}
				
			}).perform();
			
			
			load("20percentChangeOfStallingForAPeak", () -> {
				LoadUtility.randomSleep(20, 200);
			}).peak(9, 0.05).perform();
			
			ResultModel<SomeKindOfObject> resultModel = load("nothingToReturn", () -> {
				return new SomeKindOfObject();
			}).performAndGetModel();
			
			
			load("asynchronous", () -> {
				LoadUtility.sleep(30);
			}).performAsync();
			
			load("breakIteration", () -> {}).handleResult(resultHandler->{
				resultHandler.setStatus(false);
			}).throwIfFailed().perform();
			
			load("wontBeDone", () -> {
			}).perform();
			
		}

	};
	
	public void simpleLoadTest() {

		Load load = new LoadBuilder(loadScenario).amountOfThreads(10).rampup(10 * SECOND)
				.throttle(5, PER_SECOND, SHARED)
				.stopDecision(duration(1 * MINUTE)).build();

		RuntimeChart runtimeChart = new RuntimeChart();

		Result result = new ExecutionBuilder(load).storeAndConsumeResultRuntime(runtimeChart).build().execute()
				.andWait().getReportedResultFromResultFile();

		new ResultChart(result);
		result.summaryStandard().build().prettyPrint();
		
		runtimeChart.waitUntilClosed();
	}
	
	String getSecurityToken(){
		LoadUtility.sleep(50);
		return "securityToken" + System.currentTimeMillis();
	}
	
	void authenticate(String securityToken){
		LoadUtility.sleep(70);
	}
	
	class SomeKindOfObject {}
}